/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
/**
 * A loader for dynamically loading a set of packages (with dependencies) and
 * returning the global namespace variable initialized by those packages.
 *
 * @template G The type of the global namespace variable.
 */
export declare class PackageLoader<G> {
    private static readonly promises;
    /**
     * Loads the given packages and returns the loaded global namespace.
     *
     * See `TaskModelLoader.packageUrls` for more info about how to use it to
     * specify packages with dependencies.
     *
     * TODO: use a design similar to tfjs' `Platform` to provide different
     * implmentations for dynamic package loading, global, etc.
     */
    loadPackagesAndGetGlobalNamespace(namespace: string, packageUrls: Array<string[]>, curIndex?: number): Promise<G>;
    private loadPackage;
}
