"use strict";
/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var package_loader_1 = require("./package_loader");
var common_1 = require("./tasks/common");
/**
 * A loader that loads a task model.
 *
 * A task model is backed by a "source model" that does the actual work, so the
 * main task of a model loader is to load this source model and "transform" it
 * into the target task model. To do that, the client that implements the
 * `TaskModelLoader` needs to:
 *
 * - Provide a set of metadata, such as model name, runtime, etc.
 *
 * - Provide package urls and global namespace name for the source model.
 *   `TaskModelLoader` will load the packages, and return the global namespace
 *   variable provided by the source model.
 *
 * - Implement the `transformSourceModel` method to load the source model using
 *   the global namespace variable from the previous step, and transform the
 *   source model to the corresponding task model.
 *
 *   To make this process more structured, each task will have a more concrete
 *   interface of `TaskModel` with the task-specific method defined. All the
 *   models under the task will transform to this interface to make sure they
 *   have the same API signature. See tasks/image_classification/common.ts for
 *   an example.
 *
 * @template N The type of the global namespace provided by the source model.
 * @template LO The type of options used during the model loading process.
 * @template M The type of the target task model to transform to.
 */
var TaskModelLoader = /** @class */ (function () {
    function TaskModelLoader() {
        /**
         * Callback to wait for (await) after all packages are loaded.
         *
         * This is necessary for models where certain conditions need to be met before
         * using the model.
         */
        this.postPackageLoadingCallback = undefined;
        /** A loader for loading packages. */
        this.packageLoader = new package_loader_1.PackageLoader();
    }
    /**
     * Loads the task model with the given options.
     *
     * @param options Options to use when loading the model.
     * @returns The loaded instance.
     */
    TaskModelLoader.prototype.load = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var sourceModelGlobal;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.loadSourceModelGlobalNamespace(options)];
                    case 1:
                        sourceModelGlobal = _a.sent();
                        if (!this.postPackageLoadingCallback) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.postPackageLoadingCallback()];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        if (!(this.metadata.runtime === common_1.Runtime.TFJS)) return [3 /*break*/, 5];
                        return [4 /*yield*/, common_1.ensureTFJSBackend(options)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: 
                    // Load the source model using the global namespace variable loaded above.
                    return [2 /*return*/, this.transformSourceModel(sourceModelGlobal, options)];
                }
            });
        });
    };
    /**
     * Loads the global namespace of the source model by loading its packages
     * specified in the `packageUrls` field above.
     *
     * It is typically not necessary for subclasses to override this method as
     * long as they set the `packageUrls` and `sourceModelGlobalNamespace` field
     * above.
     */
    TaskModelLoader.prototype.loadSourceModelGlobalNamespace = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var packages, tfjsOptions;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        packages = [];
                        // Add TFJS dependencies for TFJS models.
                        if (this.metadata.runtime === common_1.Runtime.TFJS) {
                            tfjsOptions = options;
                            packages.push.apply(packages, common_1.getTFJSModelDependencyPackages(tfjsOptions ? tfjsOptions.backend : undefined));
                        }
                        // Load packages.
                        packages.push.apply(packages, this.packageUrls);
                        return [4 /*yield*/, this.packageLoader.loadPackagesAndGetGlobalNamespace(this.sourceModelGlobalNamespace, packages)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Loads the source model and transforms it to the corresponding task model.
     *
     * A subclass should implement this method and use the
     * `sourceModelGlobalNamespace` to load the source model. Note that the
     * subclass should *NOT* use the namespace from the import statement to
     * implement this method (e.g. the "mobilenet" variable in `import * as
     * mobilenet from
     * '@tensorflow-models/mobilenet`). Instead, only `sourceModelGlobalNamespace`
     * should be used, which should have the same type as the imported namespace.
     * The imported namespace can only be used to reference types. This makes sure
     * that the code from the source model is NOT bundled in the final binary.
     */
    TaskModelLoader.prototype.transformSourceModel = function (sourceModelGlobalNamespace, options) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw new Error('not implemented');
            });
        });
    };
    return TaskModelLoader;
}());
exports.TaskModelLoader = TaskModelLoader;
//# sourceMappingURL=task_model.js.map