/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
import { TaskModelLoader } from '../task_model';
import { Runtime } from './common';
export declare const ImageClassification: {
    MobileNet: {
        [Runtime.TFJS]: import("./image_classification/mobilenet_tfjs").MobilenetTFJSLoader;
        [Runtime.TFLITE]: import("./image_classification/mobilenet_tflite").MobilenetTFLiteLoader;
    };
    CustomModel: {
        [Runtime.TFLITE]: import("./image_classification/custom_model_tflite").ImageClassificationCustomModelTFLiteLoader;
    };
};
export declare const ObjectDetection: {
    CocoSsd: {
        [Runtime.TFJS]: import("./object_detection/cocossd_tfjs").CocoSsdTFJSLoader;
        [Runtime.TFLITE]: import("./object_detection/cocossd_tflite").CocoSsdTFLiteLoader;
    };
    CustomModel: {
        [Runtime.TFLITE]: import("./object_detection/custom_model_tflite").ObjectDetectionCustomModelTFLiteLoader;
    };
};
export declare const ImageSegmentation: {
    Deeplab: {
        [Runtime.TFJS]: import("./image_segmentation/deeplab_tfjs").DeeplapTFJSLoader;
        [Runtime.TFLITE]: import("./image_segmentation/deeplab_tflite").DeeplabTFLiteLoader;
    };
    CustomModel: {
        [Runtime.TFLITE]: import("./image_segmentation/custom_model_tflite").ImageSegmentationCustomModelTFLiteLoader;
    };
};
export declare const SentimentDetection: {
    MovieReview: {
        [Runtime.TFLITE]: import("./sentiment_detection/movie_review_tflite").MovieReviewTFLiteLoader;
    };
    Toxicity: {
        [Runtime.TFJS]: import("./sentiment_detection/toxicity_tfjs").ToxicityTFJSLoader;
    };
};
export declare const NLClassification: {
    CustomModel: {
        [Runtime.TFLITE]: import("./nl_classification/custom_model_tflite").NLClassificationCustomModelTFLiteLoader;
    };
};
export declare const QuestionAndAnswer: {
    BertQA: {
        [Runtime.TFJS]: import("./question_and_answer/bert_qa_tfjs").BertQATFJSLoader;
        [Runtime.TFLITE]: import("./question_and_answer/bert_qa_tflite").BertQATFLiteLoader;
    };
    CustomModel: {
        [Runtime.TFLITE]: import("./question_and_answer/custom_model_tflite").QuestionAnswerCustomModelTFLiteLoader;
    };
};
/**
 * Filter model loaders by runtimes.
 *
 * A model loader will be returned if its runtime matches any runtime in the
 * given array.
 */
export declare function getModelLoadersByRuntime(runtimes: Runtime[]): TaskModelLoader<{}, {}, {}>[];
/** Gets all model loaders from the index. */
export declare function getAllModelLoaders(): Array<TaskModelLoader<{}, {}, {}>>;
