"use strict";
/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("./common");
var custom_model_tflite_1 = require("./image_classification/custom_model_tflite");
var mobilenet_tfjs_1 = require("./image_classification/mobilenet_tfjs");
var mobilenet_tflite_1 = require("./image_classification/mobilenet_tflite");
var custom_model_tflite_2 = require("./image_segmentation/custom_model_tflite");
var deeplab_tfjs_1 = require("./image_segmentation/deeplab_tfjs");
var deeplab_tflite_1 = require("./image_segmentation/deeplab_tflite");
var custom_model_tflite_3 = require("./nl_classification/custom_model_tflite");
var cocossd_tfjs_1 = require("./object_detection/cocossd_tfjs");
var cocossd_tflite_1 = require("./object_detection/cocossd_tflite");
var custom_model_tflite_4 = require("./object_detection/custom_model_tflite");
var bert_qa_tfjs_1 = require("./question_and_answer/bert_qa_tfjs");
var bert_qa_tflite_1 = require("./question_and_answer/bert_qa_tflite");
var custom_model_tflite_5 = require("./question_and_answer/custom_model_tflite");
var movie_review_tflite_1 = require("./sentiment_detection/movie_review_tflite");
var toxicity_tfjs_1 = require("./sentiment_detection/toxicity_tfjs");
/**
 * The main model index.
 *
 * The index structure is: {task_name}.{model_name}.{runtime}
 *
 * Note that it is possible to programmatically generate the index from a list
 * of loaders, but it would mean that we need to define a generic type for
 * each level of the index structure (e.g. {[taskName: string]: TaskModels}).
 * This will not work well for the auto-complete system in IDEs because
 * typescript doesn't know the candidates to show from the generic types.
 *
 * For this reason, we chose to create the index manually like below. In this
 * way, typescript will infer the types based on the content statically, and
 * correctly show the candidates when doing auto-complete.
 *
 * TODO: add test to make sure loaders are located in the correct index entries.
 */
var modelIndex = (_a = {},
    _a[common_1.Task.IMAGE_CLASSIFICATION] = {
        MobileNet: (_b = {},
            _b[common_1.Runtime.TFJS] = mobilenet_tfjs_1.mobilenetTfjsLoader,
            _b[common_1.Runtime.TFLITE] = mobilenet_tflite_1.mobilenetTfliteLoader,
            _b),
        CustomModel: (_c = {},
            _c[common_1.Runtime.TFLITE] = custom_model_tflite_1.imageClassificationCustomModelTfliteLoader,
            _c),
    },
    _a[common_1.Task.OBJECT_DETECTION] = {
        CocoSsd: (_d = {},
            _d[common_1.Runtime.TFJS] = cocossd_tfjs_1.cocoSsdTfjsLoader,
            _d[common_1.Runtime.TFLITE] = cocossd_tflite_1.cocoSsdTfliteLoader,
            _d),
        CustomModel: (_e = {},
            _e[common_1.Runtime.TFLITE] = custom_model_tflite_4.objectDetectorCustomModelTfliteLoader,
            _e),
    },
    _a[common_1.Task.IMAGE_SEGMENTATION] = {
        Deeplab: (_f = {},
            _f[common_1.Runtime.TFJS] = deeplab_tfjs_1.deeplabTfjsLoader,
            _f[common_1.Runtime.TFLITE] = deeplab_tflite_1.deeplabTfliteLoader,
            _f),
        CustomModel: (_g = {},
            _g[common_1.Runtime.TFLITE] = custom_model_tflite_2.imageSegmenterCustomModelTfliteLoader,
            _g),
    },
    _a[common_1.Task.SENTIMENT_DETECTION] = {
        MovieReview: (_h = {},
            _h[common_1.Runtime.TFLITE] = movie_review_tflite_1.movieReviewTfliteLoader,
            _h),
        Toxicity: (_j = {},
            _j[common_1.Runtime.TFJS] = toxicity_tfjs_1.toxicityTfjsLoader,
            _j)
    },
    _a[common_1.Task.NL_CLASSIFICATION] = {
        CustomModel: (_k = {},
            _k[common_1.Runtime.TFLITE] = custom_model_tflite_3.nlClassifierCustomModelTfliteLoader,
            _k),
    },
    _a[common_1.Task.QUESTION_AND_ANSWER] = {
        BertQA: (_l = {},
            _l[common_1.Runtime.TFJS] = bert_qa_tfjs_1.bertQaTfjsLoader,
            _l[common_1.Runtime.TFLITE] = bert_qa_tflite_1.bertQaTfliteLoader,
            _l),
        CustomModel: (_m = {},
            _m[common_1.Runtime.TFLITE] = custom_model_tflite_5.qaCustomModelTfliteLoader,
            _m)
    },
    _a);
// Export each task individually.
exports.ImageClassification = modelIndex[common_1.Task.IMAGE_CLASSIFICATION];
exports.ObjectDetection = modelIndex[common_1.Task.OBJECT_DETECTION];
exports.ImageSegmentation = modelIndex[common_1.Task.IMAGE_SEGMENTATION];
exports.SentimentDetection = modelIndex[common_1.Task.SENTIMENT_DETECTION];
exports.NLClassification = modelIndex[common_1.Task.NL_CLASSIFICATION];
exports.QuestionAndAnswer = modelIndex[common_1.Task.QUESTION_AND_ANSWER];
/**
 * Filter model loaders by runtimes.
 *
 * A model loader will be returned if its runtime matches any runtime in the
 * given array.
 */
function getModelLoadersByRuntime(runtimes) {
    return filterModelLoaders(function (loader) { return runtimes.includes(loader.metadata.runtime); });
}
exports.getModelLoadersByRuntime = getModelLoadersByRuntime;
/** Gets all model loaders from the index. */
function getAllModelLoaders() {
    var modelLoaders = [];
    for (var _i = 0, _a = Object.keys(modelIndex); _i < _a.length; _i++) {
        var task = _a[_i];
        // tslint:disable-next-line:no-any
        var models = modelIndex[task];
        for (var _b = 0, _c = Object.keys(models); _b < _c.length; _b++) {
            var modelName = _c[_b];
            var runtimes = models[modelName];
            for (var _d = 0, _e = Object.keys(runtimes); _d < _e.length; _d++) {
                var runtime = _e[_d];
                var taskModel = runtimes[runtime];
                modelLoaders.push(taskModel);
            }
        }
    }
    return modelLoaders;
}
exports.getAllModelLoaders = getAllModelLoaders;
function filterModelLoaders(filterFn) {
    var filteredModelLoaders = [];
    for (var _i = 0, _a = getAllModelLoaders(); _i < _a.length; _i++) {
        var modelLoader = _a[_i];
        if (filterFn(modelLoader)) {
            filteredModelLoaders.push(modelLoader);
        }
    }
    return filteredModelLoaders;
}
// TODO: add more util functions as needed.
//# sourceMappingURL=all_tasks.js.map