"use strict";
/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils");
/** Default TFJS backend. */
exports.DEFAULT_TFJS_BACKEND = 'webgl';
/** Default TFJS version. */
exports.DEFAULT_TFJS_VERSION = '3.6.0';
/** All supported tasks. */
var Task;
(function (Task) {
    Task["IMAGE_CLASSIFICATION"] = "IMAGE_CLASSIFICATION";
    Task["OBJECT_DETECTION"] = "OBJECT_DETECTION";
    Task["IMAGE_SEGMENTATION"] = "IMAGE_SEGMENTATION";
    Task["SENTIMENT_DETECTION"] = "SENTIMENT_DETECTION";
    Task["NL_CLASSIFICATION"] = "NL_CLASSIFICATION";
    Task["QUESTION_AND_ANSWER"] = "QUESTION_AND_ANSWER";
})(Task = exports.Task || (exports.Task = {}));
/** All supported runtimes. */
var Runtime;
(function (Runtime) {
    Runtime["TFJS"] = "TFJS";
    Runtime["TFLITE"] = "TFLite";
    Runtime["MEDIA_PIPE"] = "MediaPipe";
})(Runtime = exports.Runtime || (exports.Runtime = {}));
/** A helper function to get the TFJS packages that a TFJS model depends on. */
function getTFJSModelDependencyPackages(backend, version) {
    if (backend === void 0) { backend = exports.DEFAULT_TFJS_BACKEND; }
    if (version === void 0) { version = exports.DEFAULT_TFJS_VERSION; }
    var packages = [
        ["https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-core@" + version],
        ["https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-converter@" + version],
    ];
    switch (backend) {
        case 'cpu':
            packages[1].push("https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-cpu@" + version);
            break;
        case 'webgl':
            packages[1].push("https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-cpu@" + version);
            packages[1].push("https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-webgl@" + version);
            break;
        case 'wasm':
            packages[1].push("https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-wasm@" + version + "/dist/tf-backend-wasm.min.js");
            break;
    }
    return packages;
}
exports.getTFJSModelDependencyPackages = getTFJSModelDependencyPackages;
/**
 * Makes sure the current tfjs backend matches the one in the given option.
 *
 * For TFJS models, this function should be called at the loading time as well
 * as before running inference.
 *
 * Users might run multiple TFJS models with different backend options in a web
 * app. Only setting the backend at the model loading time is not enough because
 * the backend might be set to another one when loading a different model. We
 * also need to call this right before running the inference.
 */
function ensureTFJSBackend(options) {
    return __awaiter(this, void 0, void 0, function () {
        var backend, global, tf;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    backend = options ? options.backend : exports.DEFAULT_TFJS_BACKEND;
                    global = utils_1.isWebWorker() ? self : window;
                    tf = global['tf'];
                    if (!tf) {
                        throw new Error('tfjs not loaded');
                    }
                    if (!(tf.getBackend() !== backend)) return [3 /*break*/, 2];
                    return [4 /*yield*/, tf.setBackend(backend)];
                case 1:
                    _a.sent();
                    _a.label = 2;
                case 2: return [2 /*return*/];
            }
        });
    });
}
exports.ensureTFJSBackend = ensureTFJSBackend;
//# sourceMappingURL=common.js.map