/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
import * as qna from '@tensorflow-models/qna';
import { TaskModelLoader } from '../../task_model';
import { Runtime, Task, TFJSModelCommonLoadingOption } from '../common';
import { QuestionAnswerer, QuestionAnswerResult } from './common';
declare type QnaNs = typeof qna;
/** Loading options. */
export interface BertQATFJSLoadingOptions extends TFJSModelCommonLoadingOption, qna.ModelConfig {
}
/**
 * Inference options.
 *
 * TODO: placeholder.
 */
export interface BertQATFJSInferenceOptions {
}
/** Loader for Q&A TFJS model. */
export declare class BertQATFJSLoader extends TaskModelLoader<QnaNs, BertQATFJSLoadingOptions, BertQATFJS> {
    readonly metadata: {
        name: string;
        description: string;
        resourceUrls: {
            'github': string;
        };
        runtime: Runtime;
        version: string;
        supportedTasks: Task[];
    };
    readonly packageUrls: string[][];
    readonly sourceModelGlobalNamespace = "qna";
    protected transformSourceModel(sourceModelGlobal: QnaNs, loadingOptions?: BertQATFJSLoadingOptions): Promise<BertQATFJS>;
}
/**
 * Pre-trained TFJS Bert Q&A model.
 *
 * Usage:
 *
 * ```js
 * // Load the model with options (optional).
 * const model = await tfTask.QuestionAndAnswer.BertQA.TFJS.load();
 *
 * // Run inference with question and context.
 * const result = await model.predict(question, context);
 * console.log(result.answers);
 *
 * // Clean up.
 * model.cleanUp();
 * ```
 *
 * Refer to `tfTask.QuestionAnswerer` for the `predict` and `cleanUp` method.
 *
 * @docextratypes [
 *   {description: 'Options for `load`', symbol: 'BertQATFJSLoadingOptions'},
 *   {description: 'Options for `predict`', symbol:
 * 'BertQATFJSInferenceOptions'}
 * ]
 *
 * @doc {heading: 'Question & Answer', subheading: 'Models'}
 */
export declare class BertQATFJS extends QuestionAnswerer<BertQATFJSInferenceOptions> {
    private qnaModel?;
    private loadingOptions?;
    constructor(qnaModel?: qna.QuestionAndAnswer, loadingOptions?: BertQATFJSLoadingOptions);
    predict(question: string, context: string, infereceOptions?: BertQATFJSInferenceOptions): Promise<QuestionAnswerResult>;
}
export declare const bertQaTfjsLoader: BertQATFJSLoader;
export {};
