"use strict";
/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var task_model_1 = require("../../task_model");
var common_1 = require("../common");
var common_2 = require("./common");
/** Loader for Q&A TFJS model. */
var BertQATFJSLoader = /** @class */ (function (_super) {
    __extends(BertQATFJSLoader, _super);
    function BertQATFJSLoader() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.metadata = {
            name: 'TFJS Bert Q&A model',
            description: 'Run Bert Q&A model with TFJS',
            resourceUrls: {
                'github': 'https://github.com/tensorflow/tfjs-models/tree/master/qna',
            },
            runtime: common_1.Runtime.TFJS,
            version: '1.0.0',
            supportedTasks: [common_1.Task.QUESTION_AND_ANSWER],
        };
        _this.packageUrls = [["https://cdn.jsdelivr.net/npm/@tensorflow-models/qna@" + _this.metadata.version]];
        _this.sourceModelGlobalNamespace = 'qna';
        return _this;
    }
    BertQATFJSLoader.prototype.transformSourceModel = function (sourceModelGlobal, loadingOptions) {
        return __awaiter(this, void 0, void 0, function () {
            var modelConfig, bertQaModel;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        modelConfig = null;
                        if (loadingOptions && loadingOptions.modelUrl) {
                            modelConfig = { modelUrl: loadingOptions.modelUrl };
                        }
                        if (loadingOptions && loadingOptions.fromTFHub != null && modelConfig) {
                            modelConfig.fromTFHub = loadingOptions.fromTFHub;
                        }
                        return [4 /*yield*/, sourceModelGlobal.load(modelConfig)];
                    case 1:
                        bertQaModel = _a.sent();
                        return [2 /*return*/, new BertQATFJS(bertQaModel, loadingOptions)];
                }
            });
        });
    };
    return BertQATFJSLoader;
}(task_model_1.TaskModelLoader));
exports.BertQATFJSLoader = BertQATFJSLoader;
/**
 * Pre-trained TFJS Bert Q&A model.
 *
 * Usage:
 *
 * ```js
 * // Load the model with options (optional).
 * const model = await tfTask.QuestionAndAnswer.BertQA.TFJS.load();
 *
 * // Run inference with question and context.
 * const result = await model.predict(question, context);
 * console.log(result.answers);
 *
 * // Clean up.
 * model.cleanUp();
 * ```
 *
 * Refer to `tfTask.QuestionAnswerer` for the `predict` and `cleanUp` method.
 *
 * @docextratypes [
 *   {description: 'Options for `load`', symbol: 'BertQATFJSLoadingOptions'},
 *   {description: 'Options for `predict`', symbol:
 * 'BertQATFJSInferenceOptions'}
 * ]
 *
 * @doc {heading: 'Question & Answer', subheading: 'Models'}
 */
var BertQATFJS = /** @class */ (function (_super) {
    __extends(BertQATFJS, _super);
    function BertQATFJS(qnaModel, loadingOptions) {
        var _this = _super.call(this) || this;
        _this.qnaModel = qnaModel;
        _this.loadingOptions = loadingOptions;
        return _this;
    }
    BertQATFJS.prototype.predict = function (question, context, infereceOptions) {
        return __awaiter(this, void 0, void 0, function () {
            var qnaResults;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.qnaModel) {
                            throw new Error('source model is not loaded');
                        }
                        return [4 /*yield*/, common_1.ensureTFJSBackend(this.loadingOptions)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.qnaModel.findAnswers(question, context)];
                    case 2:
                        qnaResults = _a.sent();
                        return [2 /*return*/, { answers: qnaResults }];
                }
            });
        });
    };
    return BertQATFJS;
}(common_2.QuestionAnswerer));
exports.BertQATFJS = BertQATFJS;
exports.bertQaTfjsLoader = new BertQATFJSLoader();
//# sourceMappingURL=bert_qa_tfjs.js.map