"use strict";
/**
 * @license
 * Copyright 2021 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var task_model_1 = require("../../task_model");
var common_1 = require("../common");
var common_2 = require("./common");
/** Loader for toxicity TFJS model. */
var ToxicityTFJSLoader = /** @class */ (function (_super) {
    __extends(ToxicityTFJSLoader, _super);
    function ToxicityTFJSLoader() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.metadata = {
            name: 'TFJS Toxicity model',
            description: 'Detect whether text contains toxic content such as ' +
                'threatening language, insults, obscenities, identity-based hate, ' +
                'or sexually explicit language.',
            resourceUrls: {
                'github': 'https://github.com/tensorflow/tfjs-models/tree/master/toxicity',
            },
            runtime: common_1.Runtime.TFJS,
            version: '1.2.2',
            supportedTasks: [common_1.Task.SENTIMENT_DETECTION],
        };
        _this.packageUrls = [["https://cdn.jsdelivr.net/npm/@tensorflow-models/toxicity@" + _this.metadata.version]];
        _this.sourceModelGlobalNamespace = 'toxicity';
        return _this;
    }
    ToxicityTFJSLoader.prototype.transformSourceModel = function (sourceModelGlobal, loadingOptions) {
        return __awaiter(this, void 0, void 0, function () {
            var toxicityModel;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, sourceModelGlobal.load(loadingOptions && loadingOptions.threshold ? loadingOptions.threshold :
                            0.85, loadingOptions && loadingOptions.toxicityLabels ?
                            loadingOptions.toxicityLabels :
                            [])];
                    case 1:
                        toxicityModel = _a.sent();
                        return [2 /*return*/, new ToxicityTFJS(toxicityModel, loadingOptions)];
                }
            });
        });
    };
    return ToxicityTFJSLoader;
}(task_model_1.TaskModelLoader));
exports.ToxicityTFJSLoader = ToxicityTFJSLoader;
/**
 * Pre-trained TFJS toxicity model.
 *
 * It detects whether text contains toxic content such as threatening language,
 * insults, obscenities, identity-based hate, or sexually explicit language.
 *
 * Usage:
 *
 * ```js
 * // Load the model with options (optional. See below for docs).
 * const model = await tfTask.SentimentDetection.Toxicity.TFJS.load();
 *
 * // Run detection on text.
 * const result = await model.predict('You are stupid');
 * console.log(result.sentimentLabels);
 *
 * // Clean up.
 * model.cleanUp();
 * ```
 *
 * By default, the model returns the prediction results of the following
 * sentiment labels:
 *
 * - toxicity
 * - severe_toxicity
 * - identity_attack
 * - insult
 * - threat
 * - sexual_explicit
 * - obscene
 *
 * Refer to `tfTask.SentimentDetection` for the `predict` and `cleanUp` method,
 * and more details about the result interface.
 *
 * @docextratypes [
 *   {description: 'Options for `load`', symbol: 'ToxicityTFJSLoadingOptions'},
 *   {description: 'Options for `predict`', symbol:
 * 'ToxicityTFJSInferenceOptions'}
 * ]
 *
 * @doc {heading: 'Sentiment Detection', subheading: 'Models'}
 */
var ToxicityTFJS = /** @class */ (function (_super) {
    __extends(ToxicityTFJS, _super);
    function ToxicityTFJS(toxicityModel, loadingOptions) {
        var _this = _super.call(this) || this;
        _this.toxicityModel = toxicityModel;
        _this.loadingOptions = loadingOptions;
        return _this;
    }
    ToxicityTFJS.prototype.predict = function (text, options) {
        return __awaiter(this, void 0, void 0, function () {
            var toxicityResults, sentimentLabels, _i, toxicityResults_1, labelResult;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.toxicityModel) {
                            throw new Error('source model is not loaded');
                        }
                        return [4 /*yield*/, common_1.ensureTFJSBackend(this.loadingOptions)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.toxicityModel.classify(text)];
                    case 2:
                        toxicityResults = _a.sent();
                        sentimentLabels = {};
                        for (_i = 0, toxicityResults_1 = toxicityResults; _i < toxicityResults_1.length; _i++) {
                            labelResult = toxicityResults_1[_i];
                            sentimentLabels[labelResult.label] = {
                                result: labelResult.results[0].match,
                                probabilities: Array.from(labelResult.results[0].probabilities),
                            };
                        }
                        return [2 /*return*/, { sentimentLabels: sentimentLabels }];
                }
            });
        });
    };
    return ToxicityTFJS;
}(common_2.SentimentDetector));
exports.ToxicityTFJS = ToxicityTFJS;
exports.toxicityTfjsLoader = new ToxicityTFJSLoader();
//# sourceMappingURL=toxicity_tfjs.js.map