/**
    * @license
    * Copyright 2021 Google LLC. All Rights Reserved.
    * Licensed under the Apache License, Version 2.0 (the "License");
    * you may not use this file except in compliance with the License.
    * You may obtain a copy of the License at
    *
    * http://www.apache.org/licenses/LICENSE-2.0
    *
    * Unless required by applicable law or agreed to in writing, software
    * distributed under the License is distributed on an "AS IS" BASIS,
    * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    * See the License for the specific language governing permissions and
    * limitations under the License.
    * =============================================================================
    */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.tfTask = {}));
}(this, (function (exports) { 'use strict';

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Whether the current environment is in web worker or not. */
  function isWebWorker() {
      return typeof window === 'undefined';
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * A loader for dynamically loading a set of packages (with dependencies) and
   * returning the global namespace variable initialized by those packages.
   *
   * @template G The type of the global namespace variable.
   */
  class PackageLoader {
      /**
       * Loads the given packages and returns the loaded global namespace.
       *
       * See `TaskModelLoader.packageUrls` for more info about how to use it to
       * specify packages with dependencies.
       *
       * TODO: use a design similar to tfjs' `Platform` to provide different
       * implmentations for dynamic package loading, global, etc.
       */
      async loadPackagesAndGetGlobalNamespace(namespace, packageUrls, curIndex = 0) {
          if (curIndex >= packageUrls.length) {
              // tslint:disable-next-line:no-any
              const global = isWebWorker() ? self : window;
              const globalNamespace = global[namespace];
              if (!globalNamespace) {
                  throw new Error(`Global namespace '${namespace}' not set after loading packages ${packageUrls}`);
              }
              return globalNamespace;
          }
          await Promise.all(packageUrls[curIndex].map(name => this.loadPackage(name)));
          return await this.loadPackagesAndGetGlobalNamespace(namespace, packageUrls, curIndex + 1);
      }
      async loadPackage(packageUrl) {
          // Don't load a package if the package is being loaded or has already been
          // loaded.
          if (PackageLoader.promises[packageUrl]) {
              return PackageLoader.promises[packageUrl];
          }
          // From webworker.
          if (isWebWorker()) {
              importScripts(packageUrl);
              PackageLoader.promises[packageUrl] = Promise.resolve();
          }
          // From webpage.
          else {
              const script = document.createElement('script');
              script.crossOrigin = 'anonymous';
              const promise = new Promise((resolve, reject) => {
                  script.onerror = () => {
                      reject();
                      document.head.removeChild(script);
                  };
                  script.onload = () => {
                      resolve();
                  };
              });
              document.head.appendChild(script);
              script.setAttribute('src', packageUrl);
              PackageLoader.promises[packageUrl] = promise;
          }
          return PackageLoader.promises[packageUrl];
      }
  }
  PackageLoader.promises = {};

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Default TFJS backend. */
  const DEFAULT_TFJS_BACKEND = 'webgl';
  /** Default TFJS version. */
  const DEFAULT_TFJS_VERSION = '3.6.0';
  /** All supported tasks. */
  var Task;
  (function (Task) {
      Task["IMAGE_CLASSIFICATION"] = "IMAGE_CLASSIFICATION";
      Task["OBJECT_DETECTION"] = "OBJECT_DETECTION";
      Task["IMAGE_SEGMENTATION"] = "IMAGE_SEGMENTATION";
      Task["SENTIMENT_DETECTION"] = "SENTIMENT_DETECTION";
      Task["NL_CLASSIFICATION"] = "NL_CLASSIFICATION";
      Task["QUESTION_AND_ANSWER"] = "QUESTION_AND_ANSWER";
  })(Task || (Task = {}));
  /** All supported runtimes. */
  var Runtime;
  (function (Runtime) {
      Runtime["TFJS"] = "TFJS";
      Runtime["TFLITE"] = "TFLite";
      Runtime["MEDIA_PIPE"] = "MediaPipe";
  })(Runtime || (Runtime = {}));
  /** A helper function to get the TFJS packages that a TFJS model depends on. */
  function getTFJSModelDependencyPackages(backend = DEFAULT_TFJS_BACKEND, version = DEFAULT_TFJS_VERSION) {
      const packages = [
          [`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-core@${version}`],
          [`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-converter@${version}`],
      ];
      switch (backend) {
          case 'cpu':
              packages[1].push(`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-cpu@${version}`);
              break;
          case 'webgl':
              packages[1].push(`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-cpu@${version}`);
              packages[1].push(`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-webgl@${version}`);
              break;
          case 'wasm':
              packages[1].push(`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-backend-wasm@${version}/dist/tf-backend-wasm.min.js`);
              break;
      }
      return packages;
  }
  /**
   * Makes sure the current tfjs backend matches the one in the given option.
   *
   * For TFJS models, this function should be called at the loading time as well
   * as before running inference.
   *
   * Users might run multiple TFJS models with different backend options in a web
   * app. Only setting the backend at the model loading time is not enough because
   * the backend might be set to another one when loading a different model. We
   * also need to call this right before running the inference.
   */
  async function ensureTFJSBackend(options) {
      const backend = options ? options.backend : DEFAULT_TFJS_BACKEND;
      // tslint:disable-next-line:no-any
      const global = isWebWorker() ? self : window;
      const tf = global['tf'];
      if (!tf) {
          throw new Error('tfjs not loaded');
      }
      if (tf.getBackend() !== backend) {
          await tf.setBackend(backend);
      }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * A loader that loads a task model.
   *
   * A task model is backed by a "source model" that does the actual work, so the
   * main task of a model loader is to load this source model and "transform" it
   * into the target task model. To do that, the client that implements the
   * `TaskModelLoader` needs to:
   *
   * - Provide a set of metadata, such as model name, runtime, etc.
   *
   * - Provide package urls and global namespace name for the source model.
   *   `TaskModelLoader` will load the packages, and return the global namespace
   *   variable provided by the source model.
   *
   * - Implement the `transformSourceModel` method to load the source model using
   *   the global namespace variable from the previous step, and transform the
   *   source model to the corresponding task model.
   *
   *   To make this process more structured, each task will have a more concrete
   *   interface of `TaskModel` with the task-specific method defined. All the
   *   models under the task will transform to this interface to make sure they
   *   have the same API signature. See tasks/image_classification/common.ts for
   *   an example.
   *
   * @template N The type of the global namespace provided by the source model.
   * @template LO The type of options used during the model loading process.
   * @template M The type of the target task model to transform to.
   */
  class TaskModelLoader {
      constructor() {
          /**
           * Callback to wait for (await) after all packages are loaded.
           *
           * This is necessary for models where certain conditions need to be met before
           * using the model.
           */
          this.postPackageLoadingCallback = undefined;
          /** A loader for loading packages. */
          this.packageLoader = new PackageLoader();
      }
      /**
       * Loads the task model with the given options.
       *
       * @param options Options to use when loading the model.
       * @returns The loaded instance.
       */
      async load(options) {
          // Load the packages to get the global namespace ready.
          const sourceModelGlobal = await this.loadSourceModelGlobalNamespace(options);
          // Wait for the callback to resolve if set.
          if (this.postPackageLoadingCallback) {
              await this.postPackageLoadingCallback();
          }
          // For tfjs models, we automatically wait for the backend to be set before
          // proceeding. Subclasses don't need to do worry about this.
          if (this.metadata.runtime === Runtime.TFJS) {
              await ensureTFJSBackend(options);
          }
          // Load the source model using the global namespace variable loaded above.
          return this.transformSourceModel(sourceModelGlobal, options);
      }
      /**
       * Loads the global namespace of the source model by loading its packages
       * specified in the `packageUrls` field above.
       *
       * It is typically not necessary for subclasses to override this method as
       * long as they set the `packageUrls` and `sourceModelGlobalNamespace` field
       * above.
       */
      async loadSourceModelGlobalNamespace(options) {
          const packages = [];
          // Add TFJS dependencies for TFJS models.
          if (this.metadata.runtime === Runtime.TFJS) {
              const tfjsOptions = options;
              packages.push(...getTFJSModelDependencyPackages(tfjsOptions ? tfjsOptions.backend : undefined));
          }
          // Load packages.
          packages.push(...this.packageUrls);
          return await this.packageLoader.loadPackagesAndGetGlobalNamespace(this.sourceModelGlobalNamespace, packages);
      }
      /**
       * Loads the source model and transforms it to the corresponding task model.
       *
       * A subclass should implement this method and use the
       * `sourceModelGlobalNamespace` to load the source model. Note that the
       * subclass should *NOT* use the namespace from the import statement to
       * implement this method (e.g. the "mobilenet" variable in `import * as
       * mobilenet from
       * '@tensorflow-models/mobilenet`). Instead, only `sourceModelGlobalNamespace`
       * should be used, which should have the same type as the imported namespace.
       * The imported namespace can only be used to reference types. This makes sure
       * that the code from the source model is NOT bundled in the final binary.
       */
      async transformSourceModel(sourceModelGlobalNamespace, options) {
          throw new Error('not implemented');
      }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all ImageClassification task models.
   *
   * @template IO The type of options used during the inference process. Different
   *     models have different inference options. See individual model for more
   *     details.
   *
   * @doc {heading: 'Image Classification', subheading: 'Base model'}
   */
  class ImageClassifier {
      /**
       * Cleans up resources if needed.
       *
       * @doc {heading: 'Image Classification', subheading: 'Base model'}
       */
      cleanUp() { }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all image classification TFLite models.
   *
   * @template T The type of inference options.
   */
  class ImageClassifierTFLite extends ImageClassifier {
      constructor(tfliteImageClassifier) {
          super();
          this.tfliteImageClassifier = tfliteImageClassifier;
      }
      async predict(img, infereceOptions) {
          if (!this.tfliteImageClassifier) {
              throw new Error('source model is not loaded');
          }
          const tfliteResults = this.tfliteImageClassifier.classify(img);
          if (!tfliteResults) {
              return { classes: [] };
          }
          const classes = tfliteResults.map(result => {
              return {
                  className: result.className,
                  score: result.probability,
              };
          });
          const finalResult = { classes };
          return finalResult;
      }
      cleanUp() {
          if (!this.tfliteImageClassifier) {
              throw new Error('source model is not loaded');
          }
          this.tfliteImageClassifier.cleanUp();
      }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for custom image classification TFLite model. */
  class ImageClassificationCustomModelTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'Image classification with TFLite models',
              description: 'An image classfier backed by the TFLite Task Library. ' +
                  'It can work with any models that meet the ' +
                  '<a href="https://www.tensorflow.org/lite/inference_with_metadata/' +
                  'task_library/image_classifier#model_compatibility_requirements" ' +
                  'target:"_blank">model requirements</a>. Try models from this ' +
                  '<a href="https://tfhub.dev/tensorflow/collections/lite/task-library/' +
                  'image-classifier/1" target="_blank">collection</a>.',
              resourceUrls: {
                  'TFLite task library': 'https://www.tensorflow.org/lite/' +
                      'inference_with_metadata/task_library/overview',
              },
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.IMAGE_CLASSIFICATION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const tfliteImageClassifier = await sourceModelGlobal.ImageClassifier.create(loadingOptions.model, loadingOptions);
          return new ICCustomModelTFLite(tfliteImageClassifier);
      }
  }
  /**
   * A custom TFLite image classification model loaded from a model url or
   * an `ArrayBuffer` in memory.
   *
   * The underlying image classifier is built on top of the [TFLite Task
   * Library](https://www.tensorflow.org/lite/inference_with_metadata/task_library/overview).
   * As a result, the custom model needs to meet the [metadata
   * requirements](https://www.tensorflow.org/lite/inference_with_metadata/task_library/image_classifier#model_compatibility_requirements).
   *
   * Usage:
   *
   * ```js
   * // Load the model from a custom url with other options (optional).
   * const model = await tfTask.ImageClassification.CustomModel.TFLite.load({
   *   model:
   * 'https://tfhub.dev/google/lite-model/aiy/vision/classifier/plants_V1/3',
   * });
   *
   * // Run inference on an image.
   * const img = document.querySelector('img');
   * const result = await model.predict(img);
   * console.log(result.classes);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ImageClassifier` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'ICCustomModelTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'ICCustomModelTFLiteInferenceOptions'}
   * ]
   *
   *
   * @doc {heading: 'Image Classification', subheading: 'Models'}
   */
  class ICCustomModelTFLite extends ImageClassifierTFLite {
  }
  const imageClassificationCustomModelTfliteLoader = new ImageClassificationCustomModelTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for mobilenet TFJS model. */
  class MobilenetTFJSLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFJS Mobilenet',
              description: 'Run mobilenet image classification model with TFJS',
              resourceUrls: {
                  'github': 'https://github.com/tensorflow/tfjs-models/tree/master/mobilenet',
              },
              runtime: Runtime.TFJS,
              version: '2.1.0',
              supportedTasks: [Task.IMAGE_CLASSIFICATION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow-models/mobilenet@${this.metadata.version}`]];
          this.sourceModelGlobalNamespace = 'mobilenet';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const mobilenetModel = await sourceModelGlobal.load(loadingOptions);
          return new MobilenetTFJS(mobilenetModel, loadingOptions);
      }
  }
  /**
   * Pre-trained TFJS mobilenet model.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * //
   * // By default, it uses mobilenet V1 with webgl backend. You can change them
   * // in the options parameter of the `load` function (see below for docs).
   * const model = await tfTask.ImageClassification.Mobilenet.TFJS.load();
   *
   * // Run inference on an image with options (optional).
   * const img = document.querySelector('img');
   * const result = await model.predict(img, {topK: 5});
   * console.log(result.classes);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ImageClassifier` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol: 'MobilenetTFJSLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'MobilenetTFJSInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Image Classification', subheading: 'Models'}
   */
  class MobilenetTFJS extends ImageClassifier {
      constructor(mobilenetModel, loadingOptions) {
          super();
          this.mobilenetModel = mobilenetModel;
          this.loadingOptions = loadingOptions;
      }
      async predict(img, infereceOptions) {
          if (!this.mobilenetModel) {
              throw new Error('source model is not loaded');
          }
          await ensureTFJSBackend(this.loadingOptions);
          const mobilenetResults = await this.mobilenetModel.classify(img, infereceOptions ? infereceOptions.topK : undefined);
          const classes = mobilenetResults.map(result => {
              return {
                  className: result.className,
                  score: result.probability,
              };
          });
          const finalResult = {
              classes,
          };
          return finalResult;
      }
  }
  const mobilenetTfjsLoader = new MobilenetTFJSLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for mobilenet TFLite model. */
  class MobilenetTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFLite Mobilenet',
              description: 'Run mobilenet image classification model with TFLite',
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.IMAGE_CLASSIFICATION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          // Construct the mobilenet model url from version and alpha.
          let mobilenetVersion = '1';
          let mobilenetAlpha = '1.0';
          if (loadingOptions) {
              if (loadingOptions.version !== undefined) {
                  mobilenetVersion = String(loadingOptions.version);
              }
              if (loadingOptions.alpha !== undefined) {
                  mobilenetAlpha = String(loadingOptions.alpha);
              }
          }
          // There is no TFLite mobilenet v2 model available other than 2_1.0.
          if (mobilenetVersion === '2' && mobilenetAlpha !== '1.0') {
              mobilenetAlpha = '1.0';
              console.warn(`No mobilenet TFLite model available for ${mobilenetVersion}_${mobilenetAlpha}. Using 2_1.0 instead.`);
          }
          // TODO: use TFHub url when CORS is correctly set.
          const url = `https://storage.googleapis.com/tfweb/models/mobilenet_v${mobilenetVersion}_${mobilenetAlpha}_224_1_metadata_1.tflite`;
          const tfliteImageClassifier = await sourceModelGlobal.ImageClassifier.create(url, loadingOptions);
          return new MobilenetTFLite(tfliteImageClassifier);
      }
  }
  /**
   * Pre-trained TFLite mobilenet image classification model.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * //
   * // By default, it uses mobilenet V1. You can change it in the options
   * // parameter of the `load` function (see below for docs).
   * const model = await tfTask.ImageClassification.Mobilenet.TFJS.load();
   *
   * // Run inference on an image.
   * const img = document.querySelector('img');
   * const result = await model.predict(img);
   * console.log(result.classes);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ImageClassifier` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'MobilenetTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`',
   * symbol: 'MobilenetTFLiteInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Image Classification', subheading: 'Models'}
   */
  class MobilenetTFLite extends ImageClassifierTFLite {
  }
  const mobilenetTfliteLoader = new MobilenetTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all ImageSegmentation task models.
   *
   * @template IO The type of options used during the inference process. Different
   *     models have different inference options. See individual model for more
   *     details.
   *
   * @doc {heading: 'Image Segmentation', subheading: 'Base model'}
   */
  class ImageSegmenter {
      /**
       * Cleans up resources if needed.
       *
       * @doc {heading: 'Image Segmentation', subheading: 'Base model'}
       */
      cleanUp() { }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all image segmentation TFLite models.
   *
   * @template T The type of inference options.
   */
  class ImageSegmenterTFLite extends ImageSegmenter {
      constructor(tfliteImageSegmenter) {
          super();
          this.tfliteImageSegmenter = tfliteImageSegmenter;
      }
      async predict(img, infereceOptions) {
          if (!this.tfliteImageSegmenter) {
              throw new Error('source model is not loaded');
          }
          const tfliteResults = this.tfliteImageSegmenter.segment(img);
          if (!tfliteResults) {
              return {
                  legend: {},
                  width: -1,
                  height: -1,
                  segmentationMap: undefined,
              };
          }
          const segmentation = tfliteResults[0];
          const legend = {};
          const colors = [];
          for (const coloredLabel of segmentation.coloredLabels) {
              legend[coloredLabel.className || coloredLabel.displayName] = coloredLabel;
              colors.push(coloredLabel);
          }
          const segmentationMap = new Uint8ClampedArray(segmentation.width * segmentation.height * 4);
          for (let i = 0; i < segmentation.categoryMask.length; i++) {
              const categoryIndex = segmentation.categoryMask[i];
              const color = colors[categoryIndex];
              segmentationMap[i * 4] = color.r;
              segmentationMap[i * 4 + 1] = color.g;
              segmentationMap[i * 4 + 2] = color.b;
              segmentationMap[i * 4 + 3] = 255;
          }
          return {
              legend,
              width: segmentation.width,
              height: segmentation.height,
              segmentationMap,
          };
      }
      cleanUp() {
          if (!this.tfliteImageSegmenter) {
              throw new Error('source model is not loaded');
          }
          this.tfliteImageSegmenter.cleanUp();
      }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for custom image segmentation TFLite model. */
  class ImageSegmentationCustomModelTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'Image segmentation with TFLite models',
              description: 'An image segmentation backed by the TFLite Task Library. ' +
                  'It can work with any models that meet the ' +
                  '<a href="https://www.tensorflow.org/lite/inference_with_metadata/' +
                  'task_library/image_segmenter#model_compatibility_requirements" ' +
                  'target="_blank">model requirements</a>.Try models from this ' +
                  '<a href="https://tfhub.dev/tensorflow/collections/lite/task-library/' +
                  'image-segmenter/1" target="_blank">collection</a>.',
              resourceUrls: {
                  'TFLite task library': 'https://www.tensorflow.org/lite/' +
                      'inference_with_metadata/task_library/overview',
              },
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.IMAGE_SEGMENTATION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const tfliteImageSegmenter = await sourceModelGlobal.ImageSegmenter.create(loadingOptions.model, loadingOptions);
          return new ISCustomModelTFLite(tfliteImageSegmenter);
      }
  }
  /**
   * A custom TFLite image segmentation model loaded from a model url or an
   * `ArrayBuffer` in memory.
   *
   * The underlying image segmenter is built on top of the [TFLite Task
   * Library](https://www.tensorflow.org/lite/inference_with_metadata/task_library/overview).
   * As a result, the custom model needs to meet the [metadata
   * requirements](https://www.tensorflow.org/lite/inference_with_metadata/task_library/image_segmenter#model_compatibility_requirements).
   *
   * Usage:
   *
   * ```js
   * // Load the model from a custom url with other options (optional).
   * const model = await tfTask.ImageSemgentation.CustomModel.TFLite.load({
   *   model:
   * 'https://tfhub.dev/tensorflow/lite-model/deeplabv3/1/metadata/2?lite-format=tflite',
   * });
   *
   * // Run inference on an image.
   * const img = document.querySelector('img');
   * const result = await model.predict(img);
   * console.log(result);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ImageSegmenter` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'ISCustomModelTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'ISCustomModelTFLiteInferenceOptions'}
   * ]
   *
   *
   * @doc {heading: 'Image Segmentation', subheading: 'Models'}
   */
  class ISCustomModelTFLite extends ImageSegmenterTFLite {
  }
  const imageSegmenterCustomModelTfliteLoader = new ImageSegmentationCustomModelTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for deeplab TFJS model. */
  class DeeplapTFJSLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFJS Deeplab',
              description: 'Run deeplab image segmentation model with TFJS',
              resourceUrls: {
                  'github': 'https://github.com/tensorflow/tfjs-models/tree/master/deeplab',
              },
              runtime: Runtime.TFJS,
              version: '0.2.1',
              supportedTasks: [Task.IMAGE_SEGMENTATION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow-models/deeplab@${this.metadata.version}`]];
          this.sourceModelGlobalNamespace = 'deeplab';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const options = Object.assign({}, loadingOptions) ||
              { backend: 'webgl' };
          if (options.base == null) {
              options.base = 'pascal';
          }
          if (options.quantizationBytes == null) {
              options.quantizationBytes = 2;
          }
          const deeplabModel = await sourceModelGlobal.load(options);
          return new DeeplabTFJS(deeplabModel, options);
      }
  }
  /**
   * Pre-trained TFJS depelab model.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * //
   * // By default, it uses base='pascal' and quantizationBytes=2 with webgl
   * // backend. You can change them in the options parameter of the `load`
   * // function (see below for docs).
   * const model = await tfTask.ImageSegmentation.Deeplab.TFJS.load();
   *
   * // Run inference on an image with options (optional).
   * const img = document.querySelector('img');
   * const result = await model.predict(img);
   * console.log(result);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ImageSegmenter` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol: 'DeeplabTFJSLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'DeeplabTFJSInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Image Segmentation', subheading: 'Models'}
   */
  class DeeplabTFJS extends ImageSegmenter {
      constructor(deeplabModel, loadingOptions) {
          super();
          this.deeplabModel = deeplabModel;
          this.loadingOptions = loadingOptions;
      }
      async predict(img, infereceOptions) {
          if (!this.deeplabModel) {
              throw new Error('source model is not loaded');
          }
          await ensureTFJSBackend(this.loadingOptions);
          const deeplabResult = await this.deeplabModel.segment(img, infereceOptions);
          const legend = {};
          for (const name of Object.keys(deeplabResult.legend)) {
              const colors = deeplabResult.legend[name];
              legend[name] = {
                  r: colors[0],
                  g: colors[1],
                  b: colors[2],
              };
          }
          return {
              legend,
              width: deeplabResult.width,
              height: deeplabResult.height,
              segmentationMap: deeplabResult.segmentationMap,
          };
      }
      cleanUp() {
          if (!this.deeplabModel) {
              throw new Error('source model is not loaded');
          }
          this.deeplabModel.dispose();
      }
  }
  const deeplabTfjsLoader = new DeeplapTFJSLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for deeplab TFLite model. */
  class DeeplabTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFLite Deeplab',
              description: 'Run Deeplab image segmentation model with TFLite',
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.IMAGE_SEGMENTATION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const url = 'https://tfhub.dev/tensorflow/lite-model/' +
              'deeplabv3/1/metadata/2?lite-format=tflite';
          const tfliteImageSegmenter = await sourceModelGlobal.ImageSegmenter.create(url, loadingOptions);
          return new DeeplabTFLite(tfliteImageSegmenter);
      }
  }
  /**
   * Pre-trained TFLite deeplab image segmentation model.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * const model = await tfTask.ImageSegmentation.Deeplab.TFLite.load();
   *
   * // Run inference on an image.
   * const img = document.querySelector('img');
   * const result = await model.predict(img);
   * console.log(result);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ImageSegmenter` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'DeeplabTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`',
   * symbol: 'DeeplabTFLiteInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Image Segmentation', subheading: 'Models'}
   */
  class DeeplabTFLite extends ImageSegmenterTFLite {
  }
  const deeplabTfliteLoader = new DeeplabTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all NLClassification task models.
   *
   * @template IO The type of options used during the inference process. Different
   *     models have different inference options. See individual model for more
   *     details.
   *
   * @doc {heading: 'NL Classification', subheading: 'Base model'}
   */
  class NLClassifier {
      /**
       * Cleans up resources if needed.
       *
       * @doc {heading: 'NL Classification', subheading: 'Base model'}
       */
      cleanUp() { }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all NL classification TFLite models.
   *
   * @template T The type of inference options.
   */
  class NLClassifierTFLite extends NLClassifier {
      constructor(tfliteNLClassifier) {
          super();
          this.tfliteNLClassifier = tfliteNLClassifier;
      }
      async predict(text, infereceOptions) {
          if (!this.tfliteNLClassifier) {
              throw new Error('source model is not loaded');
          }
          const tfliteResults = this.tfliteNLClassifier.classify(text);
          if (!tfliteResults) {
              return { classes: [] };
          }
          const classes = tfliteResults.map(result => {
              return {
                  className: result.className,
                  score: result.probability,
              };
          });
          const finalResult = { classes };
          return finalResult;
      }
      cleanUp() {
          if (!this.tfliteNLClassifier) {
              throw new Error('source model is not loaded');
          }
          this.tfliteNLClassifier.cleanUp();
      }
  }
  /** Merges the given options with the default NLClassifier options. */
  function getNLClassifierOptions(options) {
      const nlclassifierOptions = {
          inputTensorIndex: 0,
          outputScoreTensorIndex: 0,
          outputLabelTensorIndex: -1,
          inputTensorName: 'INPUT',
          outputScoreTensorName: 'OUTPUT_SCORE',
          outputLabelTensorName: 'OUTPUT_LABEL',
      };
      if (!options) {
          return nlclassifierOptions;
      }
      if (options.inputTensorIndex != null) {
          nlclassifierOptions.inputTensorIndex = options.inputTensorIndex;
      }
      if (options.outputScoreTensorIndex != null) {
          nlclassifierOptions.outputScoreTensorIndex = options.outputScoreTensorIndex;
      }
      if (options.outputLabelTensorIndex != null) {
          nlclassifierOptions.outputLabelTensorIndex = options.outputLabelTensorIndex;
      }
      if (options.inputTensorName != null) {
          nlclassifierOptions.inputTensorName = options.inputTensorName;
      }
      if (options.outputScoreTensorName != null) {
          nlclassifierOptions.outputScoreTensorName = options.outputScoreTensorName;
      }
      if (options.outputLabelTensorName != null) {
          nlclassifierOptions.outputLabelTensorName = options.outputLabelTensorName;
      }
      return nlclassifierOptions;
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for custom nl classification TFLite model. */
  class NLClassificationCustomModelTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'Natural language classification with TFLite models',
              description: 'A natural language detector backed by the NLClassifier in ' +
                  'TFLite Task Library. ' +
                  'It can work with any models that meet the ' +
                  '<a href="https://www.tensorflow.org/lite/inference_with_metadata/' +
                  'task_library/nl_classifier#model_compatibility_requirements" ' +
                  'target="_blank">model requirements</a>.',
              resourceUrls: {
                  'TFLite task library': 'https://www.tensorflow.org/lite/' +
                      'inference_with_metadata/task_library/overview',
              },
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.NL_CLASSIFICATION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const tfliteNLClassifier = await sourceModelGlobal.NLClassifier.create(loadingOptions.model, getNLClassifierOptions(loadingOptions));
          return new NCCustomModelTFLite(tfliteNLClassifier);
      }
  }
  /**
   * A custom TFLite natural language classification model loaded from a model url
   * or an `ArrayBuffer` in memory.
   *
   * The underlying NL classifier is built on top of the NLClassifier in
   * [TFLite Task
   * Library](https://www.tensorflow.org/lite/inference_with_metadata/task_library/overview).
   * As a result, the custom model needs to meet the [metadata
   * requirements](https://www.tensorflow.org/lite/inference_with_metadata/task_library/nl_classifier#model_compatibility_requirements).
   *
   * Usage:
   *
   * ```js
   * // Load the model from a custom url with other options (optional).
   * const model = await tfTask.NLClassification.CustomModel.TFLite.load({
   *   model:
   * 'https://storage.googleapis.com/download.tensorflow.org/models/tflite/text_classification/text_classification_v2.tflite',
   * });
   *
   * // Run inference on text.
   * const result = await model.predict('This is a great movie!');
   * console.log(result.classes);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.NLClassifier` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'NCCustomModelTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'NCCustomModelTFLiteInferenceOptions'}
   * ]
   *
   *
   * @doc {heading: 'NL Classification', subheading: 'Models'}
   */
  class NCCustomModelTFLite extends NLClassifierTFLite {
  }
  const nlClassifierCustomModelTfliteLoader = new NLClassificationCustomModelTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all ObjectDetection task models.
   *
   * @template IO The type of options used during the inference process. Different
   *     models have different inference options. See individual model for more
   *     details.
   *
   * @doc {heading: 'Object Detection', subheading: 'Base model'}
   */
  class ObjectDetector {
      /**
       * Cleans up resources if needed.
       *
       * @doc {heading: 'Object Detection', subheading: 'Base model'}
       */
      cleanUp() { }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for cocossd TFJS model. */
  class CocoSsdTFJSLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFJS COCO-SSD',
              description: 'Run COCO-SSD object detection model with TFJS',
              resourceUrls: {
                  'github': 'https://github.com/tensorflow/tfjs-models/tree/master/coco-ssd',
              },
              runtime: Runtime.TFJS,
              version: '2.2.2',
              supportedTasks: [Task.OBJECT_DETECTION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow-models/coco-ssd@${this.metadata.version}`]];
          this.sourceModelGlobalNamespace = 'cocoSsd';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const cocoSsdModel = await sourceModelGlobal.load(loadingOptions);
          return new CocoSsdTFJS(cocoSsdModel, loadingOptions);
      }
  }
  /**
   * Pre-trained TFJS coco-ssd model.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * //
   * // By default, it uses lite_mobilenet_v2 as the base model with webgl
   * // backend. You can change them in the `options` parameter of the `load`
   * // function (see below for docs).
   * const model = await tfTask.ObjectDetection.CocoSsd.TFJS.load();
   *
   * // Run detection on an image with options (optional).
   * const img = document.querySelector('img');
   * const result = await model.predict(img, {numMaxBoxes: 5});
   * console.log(result.objects);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ObjectDetector` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol: 'CocoSsdTFJSLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'CocoSsdTFJSInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Object Detection', subheading: 'Models'}
   */
  class CocoSsdTFJS extends ObjectDetector {
      constructor(cocoSsdModel, loadingOptions) {
          super();
          this.cocoSsdModel = cocoSsdModel;
          this.loadingOptions = loadingOptions;
      }
      async predict(img, infereceOptions) {
          if (!this.cocoSsdModel) {
              throw new Error('source model is not loaded');
          }
          await ensureTFJSBackend(this.loadingOptions);
          const cocoSsdResults = await this.cocoSsdModel.detect(img, infereceOptions ? infereceOptions.maxNumBoxes : undefined, infereceOptions ? infereceOptions.minScore : undefined);
          const objects = cocoSsdResults.map(result => {
              return {
                  boundingBox: {
                      originX: result.bbox[0],
                      originY: result.bbox[1],
                      width: result.bbox[2],
                      height: result.bbox[3],
                  },
                  className: result.class,
                  score: result.score,
              };
          });
          const finalResult = {
              objects,
          };
          return finalResult;
      }
      cleanUp() {
          if (!this.cocoSsdModel) {
              throw new Error('source model is not loaded');
          }
          return this.cocoSsdModel.dispose();
      }
  }
  const cocoSsdTfjsLoader = new CocoSsdTFJSLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all object detection TFLite models.
   *
   * @template T The type of inference options.
   */
  class ObjectDetectorTFLite extends ObjectDetector {
      constructor(tfliteObjectDetector) {
          super();
          this.tfliteObjectDetector = tfliteObjectDetector;
      }
      async predict(img, infereceOptions) {
          if (!this.tfliteObjectDetector) {
              throw new Error('source model is not loaded');
          }
          const tfliteResults = this.tfliteObjectDetector.detect(img);
          if (!tfliteResults) {
              return { objects: [] };
          }
          const objects = tfliteResults.map(result => {
              return {
                  boundingBox: result.boundingBox,
                  className: result.classes[0].className,
                  score: result.classes[0].probability,
              };
          });
          const finalResult = { objects };
          return finalResult;
      }
      cleanUp() {
          if (!this.tfliteObjectDetector) {
              throw new Error('source model is not loaded');
          }
          this.tfliteObjectDetector.cleanUp();
      }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for cocossd TFLite model. */
  class CocoSsdTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFLite COCO-SSD',
              description: 'Run COCO-SSD object detection model with TFLite',
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.OBJECT_DETECTION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const url = 'https://tfhub.dev/tensorflow/lite-model/' +
              'ssd_mobilenet_v1/1/metadata/2?lite-format=tflite';
          const tfliteObjectDetector = await sourceModelGlobal.ObjectDetector.create(url, loadingOptions);
          return new CocoSsdTFLite(tfliteObjectDetector);
      }
  }
  /**
   * Pre-trained TFLite coco-ssd object detection model.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * const model = await tfTask.ObjectDetection.CocoSsd.TFLite.load();
   *
   * // Run inference on an image.
   * const img = document.querySelector('img');
   * const result = await model.predict(img);
   * console.log(result.objects);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ObjectDetector` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'CocoSsdTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`',
   * symbol: 'CocoSsdTFLiteInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Object Detection', subheading: 'Models'}
   */
  class CocoSsdTFLite extends ObjectDetectorTFLite {
  }
  const cocoSsdTfliteLoader = new CocoSsdTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for custom object detection TFLite model. */
  class ObjectDetectionCustomModelTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'Object detection with TFLite models',
              description: 'An object detector backed by the TFLite Task Library. ' +
                  'It can work with any models that meet the ' +
                  '<a href="https://www.tensorflow.org/lite/inference_with_metadata/' +
                  'task_library/object_detector#model_compatibility_requirements" ' +
                  'target="_blank">model requirements</a>. Try models from this ' +
                  '<a href="https://tfhub.dev/tensorflow/collections/lite/task-library/' +
                  'object-detector/1" target="_blank">collection</a>.',
              resourceUrls: {
                  'TFLite task library': 'https://www.tensorflow.org/lite/' +
                      'inference_with_metadata/task_library/overview',
              },
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.OBJECT_DETECTION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const tfliteObjectDetector = await sourceModelGlobal.ObjectDetector.create(loadingOptions.model, loadingOptions);
          return new ODCustomModelTFLite(tfliteObjectDetector);
      }
  }
  /**
   * A custom TFLite object detection model loaded from a model url or an
   * `ArrayBuffer` in memory.
   *
   * The underlying object detector is built on top of the [TFLite Task
   * Library](https://www.tensorflow.org/lite/inference_with_metadata/task_library/overview).
   * As a result, the custom model needs to meet the [metadata
   * requirements](https://www.tensorflow.org/lite/inference_with_metadata/task_library/object_detector#model_compatibility_requirements).
   *
   * Usage:
   *
   * ```js
   * // Load the model from a custom url with other options (optional).
   * const model = await tfTask.ObjectDetection.CustomModel.TFLite.load({
   *   model:
   * 'https://tfhub.dev/tensorflow/lite-model/ssd_mobilenet_v1/1/metadata/2?lite-format=tflite',
   * });
   *
   * // Run inference on an image.
   * const img = document.querySelector('img');
   * const result = await model.predict(img);
   * console.log(result.objects);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.ObjectDetector` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'ODCustomModelTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'ODCustomModelTFLiteInferenceOptions'}
   * ]
   *
   *
   * @doc {heading: 'Object Detection', subheading: 'Models'}
   */
  class ODCustomModelTFLite extends ObjectDetectorTFLite {
  }
  const objectDetectorCustomModelTfliteLoader = new ObjectDetectionCustomModelTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all Q&A task models.
   *
   * @template IO The type of options used during the inference process. Different
   *     models have different inference options. See individual model for more
   *     details.
   *
   * @doc {heading: 'Question & Answer', subheading: 'Base model'}
   */
  class QuestionAnswerer {
      /**
       * Cleans up resources if needed.
       *
       * @doc {heading: 'Question & Answer', subheading: 'Base model'}
       */
      cleanUp() { }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for Q&A TFJS model. */
  class BertQATFJSLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFJS Bert Q&A model',
              description: 'Run Bert Q&A model with TFJS',
              resourceUrls: {
                  'github': 'https://github.com/tensorflow/tfjs-models/tree/master/qna',
              },
              runtime: Runtime.TFJS,
              version: '1.0.0',
              supportedTasks: [Task.QUESTION_AND_ANSWER],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow-models/qna@${this.metadata.version}`]];
          this.sourceModelGlobalNamespace = 'qna';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          let modelConfig = null;
          if (loadingOptions && loadingOptions.modelUrl) {
              modelConfig = { modelUrl: loadingOptions.modelUrl };
          }
          if (loadingOptions && loadingOptions.fromTFHub != null && modelConfig) {
              modelConfig.fromTFHub = loadingOptions.fromTFHub;
          }
          const bertQaModel = await sourceModelGlobal.load(modelConfig);
          return new BertQATFJS(bertQaModel, loadingOptions);
      }
  }
  /**
   * Pre-trained TFJS Bert Q&A model.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * const model = await tfTask.QuestionAndAnswer.BertQA.TFJS.load();
   *
   * // Run inference with question and context.
   * const result = await model.predict(question, context);
   * console.log(result.answers);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.QuestionAnswerer` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol: 'BertQATFJSLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'BertQATFJSInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Question & Answer', subheading: 'Models'}
   */
  class BertQATFJS extends QuestionAnswerer {
      constructor(qnaModel, loadingOptions) {
          super();
          this.qnaModel = qnaModel;
          this.loadingOptions = loadingOptions;
      }
      async predict(question, context, infereceOptions) {
          if (!this.qnaModel) {
              throw new Error('source model is not loaded');
          }
          await ensureTFJSBackend(this.loadingOptions);
          const qnaResults = await this.qnaModel.findAnswers(question, context);
          return { answers: qnaResults };
      }
  }
  const bertQaTfjsLoader = new BertQATFJSLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all Q&A TFLite models.
   *
   * @template T The type of inference options.
   */
  class QuestionAnswererTFLite extends QuestionAnswerer {
      constructor(tfliteQuestionAnswerer) {
          super();
          this.tfliteQuestionAnswerer = tfliteQuestionAnswerer;
      }
      async predict(question, context, infereceOptions) {
          if (!this.tfliteQuestionAnswerer) {
              throw new Error('source model is not loaded');
          }
          // In TFLite task library, context is the first parameter.
          const tfliteResults = this.tfliteQuestionAnswerer.answer(context, question);
          const answers = tfliteResults.map(result => {
              return {
                  text: result.text,
                  startIndex: result.pos.start,
                  endIndex: result.pos.end,
                  score: result.pos.logit,
              };
          });
          return { answers };
      }
      cleanUp() {
          if (!this.tfliteQuestionAnswerer) {
              throw new Error('source model is not loaded');
          }
          this.tfliteQuestionAnswerer.cleanUp();
      }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for Bert Q&A TFLite model. */
  class BertQATFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFLite Bert Q&A model',
              description: 'Run Bert Q&A model with TFLite',
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.QUESTION_AND_ANSWER],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const url = 'https://tfhub.dev/tensorflow/lite-model/' +
              'mobilebert/1/metadata/1?lite-format=tflite';
          const tfliteQa = await sourceModelGlobal.BertQuestionAnswerer.create(url);
          return new BertQATFLite(tfliteQa);
      }
  }
  /**
   * Pre-trained TFLite Bert Q&A model.
   *
   * Usage:
   *
   * ```js
   * // Load the model.
   * const model = await tfTask.QuestionAndAnswer.BertQA.TFLite.load();
   *
   * // Run inference on an image.
   * const result = await model.predict(question, context);
   * console.log(result);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.QuestionAnswerer` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'BertQATFLiteLoadingOptions'},
   *   {description: 'Options for `predict`',
   * symbol: 'BertQATFLiteInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Question & Answer', subheading: 'Models'}
   */
  class BertQATFLite extends QuestionAnswererTFLite {
  }
  const bertQaTfliteLoader = new BertQATFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for custom Q&A TFLite model. */
  class QuestionAnswerCustomModelTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'Question&Answer with TFLite models',
              description: 'A QuestionAnswerer backed by the TFLite Task Library. ' +
                  'It can work with any models that meet the ' +
                  '<a href="https://www.tensorflow.org/lite/inference_with_metadata/' +
                  'task_library/bert_question_answerer#model_compatibility_requirements" ' +
                  'target="_blank">model requirements</a>. Try models from this ' +
                  '<a href="https://tfhub.dev/tensorflow/collections/lite/task-library/' +
                  'bert-question-answerer/1" target="_blank">collection</a>.',
              resourceUrls: {
                  'TFLite task library': 'https://www.tensorflow.org/lite/' +
                      'inference_with_metadata/task_library/overview',
              },
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.QUESTION_AND_ANSWER],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const tfliteQa = await sourceModelGlobal.BertQuestionAnswerer.create(loadingOptions.model);
          return new QACustomModelTFLite(tfliteQa);
      }
  }
  /**
   * A custom TFLite Q&A model loaded from a model url or an `ArrayBuffer` in
   * memory.
   *
   * The underlying question answerer is built on top of the [TFLite Task
   * Library](https://www.tensorflow.org/lite/inference_with_metadata/task_library/overview).
   * As a result, the custom model needs to meet the [metadata
   * requirements](https://www.tensorflow.org/lite/inference_with_metadata/task_library/bert_question_answerer#model_compatibility_requirements).
   *
   * Usage:
   *
   * ```js
   * // Load the model from a custom url.
   * const model = await tfTask.QuestionAndAnswer.CustomModel.TFLite.load({
   *   model:
   * 'https://tfhub.dev/tensorflow/lite-model/mobilebert/1/metadata/1?lite-format=tflite',
   * });
   *
   * // Run inference with question and context.
   * const result = await model.predict(question, context);
   * console.log(result.answers);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * Refer to `tfTask.QuestionAnswerer` for the `predict` and `cleanUp` method.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'QACustomModelTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'QACustomModelTFLiteInferenceOptions'}
   * ]
   *
   *
   * @doc {heading: 'Question & Answer', subheading: 'Models'}
   */
  class QACustomModelTFLite extends QuestionAnswererTFLite {
  }
  const qaCustomModelTfliteLoader = new QuestionAnswerCustomModelTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The base class for all SentimentDetection task models.
   *
   * @template IO The type of options used during the inference process. Different
   *     models have different inference options. See individual model for more
   *     details.
   *
   * @doc {heading: 'Sentiment Detection', subheading: 'Base model'}
   */
  class SentimentDetector {
      /**
       * Cleans up resources if needed.
       *
       * @doc {heading: 'Sentiment Detection', subheading: 'Base model'}
       */
      cleanUp() { }
  }

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  const DEFAULT_THRESHOLD = 0.5;
  /** Loader for cocossd TFLite model. */
  class MovieReviewTFLiteLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFLite movie review model',
              description: 'Run a movie review model with TFLite and output ' +
                  'the probabilities of whether the review is positive or negetive.',
              runtime: Runtime.TFLITE,
              version: '0.0.1-alpha.3',
              supportedTasks: [Task.SENTIMENT_DETECTION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow/tfjs-tflite@${this.metadata.version}/dist/tf-tflite.min.js`]];
          this.sourceModelGlobalNamespace = 'tflite';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const url = 'https://storage.googleapis.com/tfweb/models/' +
              'movie_review_sentiment_classification.tflite';
          const tfliteNLClassifier = await sourceModelGlobal.NLClassifier.create(url, getNLClassifierOptions());
          const threshold = loadingOptions && loadingOptions.threshold != null ?
              loadingOptions.threshold :
              DEFAULT_THRESHOLD;
          return new MovieReviewTFLite(tfliteNLClassifier, threshold);
      }
  }
  /**
   * Pre-trained TFLite movie review sentiment detection model.
   *
   * It detects whether the review text is positive or negetive.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional).
   * const model = await tfTask.SentimentDetection.MovieReview.TFLite.load();
   *
   * // Run inference on a review text.
   * const result = await model.predict('This is a great movie!');
   * console.log(result.sentimentLabels);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * The model returns the prediction results of the following sentiment labels:
   *
   * - positive
   * - negative
   *
   * Refer to `tfTask.SentimentDetector` for the `predict` and `cleanUp` method,
   * and more details about the result interface.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol:
   * 'MovieReviewTFLiteLoadingOptions'},
   *   {description: 'Options for `predict`',
   * symbol: 'MovieReviewTFLiteInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Sentiment Detection', subheading: 'Models'}
   */
  class MovieReviewTFLite extends SentimentDetector {
      constructor(tfliteNLClassifier, threshold) {
          super();
          this.tfliteNLClassifier = tfliteNLClassifier;
          this.threshold = threshold;
      }
      async predict(text, options) {
          if (!this.tfliteNLClassifier) {
              throw new Error('source model is not loaded');
          }
          const tfliteResults = this.tfliteNLClassifier.classify(text);
          if (!tfliteResults) {
              return { sentimentLabels: {} };
          }
          const negativeProbability = tfliteResults[0].probability;
          const positiveProbability = tfliteResults[1].probability;
          let positiveResult = null;
          let negativeResult = null;
          if (Math.max(negativeProbability, positiveProbability) > this.threshold) {
              positiveResult = positiveProbability > negativeProbability;
              negativeResult = positiveProbability < negativeProbability;
          }
          return {
              sentimentLabels: {
                  'positive': {
                      result: positiveResult,
                      probabilities: [negativeProbability, positiveProbability],
                  },
                  'negative': {
                      result: negativeResult,
                      probabilities: [positiveProbability, negativeProbability],
                  }
              }
          };
      }
  }
  const movieReviewTfliteLoader = new MovieReviewTFLiteLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /** Loader for toxicity TFJS model. */
  class ToxicityTFJSLoader extends TaskModelLoader {
      constructor() {
          super(...arguments);
          this.metadata = {
              name: 'TFJS Toxicity model',
              description: 'Detect whether text contains toxic content such as ' +
                  'threatening language, insults, obscenities, identity-based hate, ' +
                  'or sexually explicit language.',
              resourceUrls: {
                  'github': 'https://github.com/tensorflow/tfjs-models/tree/master/toxicity',
              },
              runtime: Runtime.TFJS,
              version: '1.2.2',
              supportedTasks: [Task.SENTIMENT_DETECTION],
          };
          this.packageUrls = [[`https://cdn.jsdelivr.net/npm/@tensorflow-models/toxicity@${this.metadata.version}`]];
          this.sourceModelGlobalNamespace = 'toxicity';
      }
      async transformSourceModel(sourceModelGlobal, loadingOptions) {
          const toxicityModel = await sourceModelGlobal.load(loadingOptions && loadingOptions.threshold ? loadingOptions.threshold :
              0.85, loadingOptions && loadingOptions.toxicityLabels ?
              loadingOptions.toxicityLabels :
              []);
          return new ToxicityTFJS(toxicityModel, loadingOptions);
      }
  }
  /**
   * Pre-trained TFJS toxicity model.
   *
   * It detects whether text contains toxic content such as threatening language,
   * insults, obscenities, identity-based hate, or sexually explicit language.
   *
   * Usage:
   *
   * ```js
   * // Load the model with options (optional. See below for docs).
   * const model = await tfTask.SentimentDetection.Toxicity.TFJS.load();
   *
   * // Run detection on text.
   * const result = await model.predict('You are stupid');
   * console.log(result.sentimentLabels);
   *
   * // Clean up.
   * model.cleanUp();
   * ```
   *
   * By default, the model returns the prediction results of the following
   * sentiment labels:
   *
   * - toxicity
   * - severe_toxicity
   * - identity_attack
   * - insult
   * - threat
   * - sexual_explicit
   * - obscene
   *
   * Refer to `tfTask.SentimentDetection` for the `predict` and `cleanUp` method,
   * and more details about the result interface.
   *
   * @docextratypes [
   *   {description: 'Options for `load`', symbol: 'ToxicityTFJSLoadingOptions'},
   *   {description: 'Options for `predict`', symbol:
   * 'ToxicityTFJSInferenceOptions'}
   * ]
   *
   * @doc {heading: 'Sentiment Detection', subheading: 'Models'}
   */
  class ToxicityTFJS extends SentimentDetector {
      constructor(toxicityModel, loadingOptions) {
          super();
          this.toxicityModel = toxicityModel;
          this.loadingOptions = loadingOptions;
      }
      async predict(text, options) {
          if (!this.toxicityModel) {
              throw new Error('source model is not loaded');
          }
          await ensureTFJSBackend(this.loadingOptions);
          const toxicityResults = await this.toxicityModel.classify(text);
          const sentimentLabels = {};
          for (const labelResult of toxicityResults) {
              sentimentLabels[labelResult.label] = {
                  result: labelResult.results[0].match,
                  probabilities: Array.from(labelResult.results[0].probabilities),
              };
          }
          return { sentimentLabels };
      }
  }
  const toxicityTfjsLoader = new ToxicityTFJSLoader();

  /**
   * @license
   * Copyright 2021 Google LLC. All Rights Reserved.
   * Licensed under the Apache License, Version 2.0 (the "License");
   * you may not use this file except in compliance with the License.
   * You may obtain a copy of the License at
   *
   * http://www.apache.org/licenses/LICENSE-2.0
   *
   * Unless required by applicable law or agreed to in writing, software
   * distributed under the License is distributed on an "AS IS" BASIS,
   * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   * See the License for the specific language governing permissions and
   * limitations under the License.
   * =============================================================================
   */
  /**
   * The main model index.
   *
   * The index structure is: {task_name}.{model_name}.{runtime}
   *
   * Note that it is possible to programmatically generate the index from a list
   * of loaders, but it would mean that we need to define a generic type for
   * each level of the index structure (e.g. {[taskName: string]: TaskModels}).
   * This will not work well for the auto-complete system in IDEs because
   * typescript doesn't know the candidates to show from the generic types.
   *
   * For this reason, we chose to create the index manually like below. In this
   * way, typescript will infer the types based on the content statically, and
   * correctly show the candidates when doing auto-complete.
   *
   * TODO: add test to make sure loaders are located in the correct index entries.
   */
  const modelIndex = {
      [Task.IMAGE_CLASSIFICATION]: {
          MobileNet: {
              [Runtime.TFJS]: mobilenetTfjsLoader,
              [Runtime.TFLITE]: mobilenetTfliteLoader,
          },
          CustomModel: {
              [Runtime.TFLITE]: imageClassificationCustomModelTfliteLoader,
          },
      },
      [Task.OBJECT_DETECTION]: {
          CocoSsd: {
              [Runtime.TFJS]: cocoSsdTfjsLoader,
              [Runtime.TFLITE]: cocoSsdTfliteLoader,
          },
          CustomModel: {
              [Runtime.TFLITE]: objectDetectorCustomModelTfliteLoader,
          },
      },
      [Task.IMAGE_SEGMENTATION]: {
          Deeplab: {
              [Runtime.TFJS]: deeplabTfjsLoader,
              [Runtime.TFLITE]: deeplabTfliteLoader,
          },
          CustomModel: {
              [Runtime.TFLITE]: imageSegmenterCustomModelTfliteLoader,
          },
      },
      [Task.SENTIMENT_DETECTION]: {
          MovieReview: {
              [Runtime.TFLITE]: movieReviewTfliteLoader,
          },
          Toxicity: {
              [Runtime.TFJS]: toxicityTfjsLoader,
          }
      },
      [Task.NL_CLASSIFICATION]: {
          CustomModel: {
              [Runtime.TFLITE]: nlClassifierCustomModelTfliteLoader,
          },
      },
      [Task.QUESTION_AND_ANSWER]: {
          BertQA: {
              [Runtime.TFJS]: bertQaTfjsLoader,
              [Runtime.TFLITE]: bertQaTfliteLoader,
          },
          CustomModel: {
              [Runtime.TFLITE]: qaCustomModelTfliteLoader,
          }
      },
  };
  // Export each task individually.
  const ImageClassification = modelIndex[Task.IMAGE_CLASSIFICATION];
  const ObjectDetection = modelIndex[Task.OBJECT_DETECTION];
  const ImageSegmentation = modelIndex[Task.IMAGE_SEGMENTATION];
  const SentimentDetection = modelIndex[Task.SENTIMENT_DETECTION];
  const NLClassification = modelIndex[Task.NL_CLASSIFICATION];
  const QuestionAndAnswer = modelIndex[Task.QUESTION_AND_ANSWER];
  /**
   * Filter model loaders by runtimes.
   *
   * A model loader will be returned if its runtime matches any runtime in the
   * given array.
   */
  function getModelLoadersByRuntime(runtimes) {
      return filterModelLoaders((loader) => runtimes.includes(loader.metadata.runtime));
  }
  /** Gets all model loaders from the index. */
  function getAllModelLoaders() {
      const modelLoaders = [];
      for (const task of Object.keys(modelIndex)) {
          // tslint:disable-next-line:no-any
          const models = modelIndex[task];
          for (const modelName of Object.keys(models)) {
              const runtimes = models[modelName];
              for (const runtime of Object.keys(runtimes)) {
                  const taskModel = runtimes[runtime];
                  modelLoaders.push(taskModel);
              }
          }
      }
      return modelLoaders;
  }
  function filterModelLoaders(filterFn) {
      const filteredModelLoaders = [];
      for (const modelLoader of getAllModelLoaders()) {
          if (filterFn(modelLoader)) {
              filteredModelLoaders.push(modelLoader);
          }
      }
      return filteredModelLoaders;
  }
  // TODO: add more util functions as needed.

  /** @license See the LICENSE file. */
  // This code is auto-generated, do not modify this file!
  const version = '0.0.1-alpha.8';

  exports.ICCustomModelTFLite = ICCustomModelTFLite;
  exports.ImageClassification = ImageClassification;
  exports.ImageClassificationCustomModelTFLiteLoader = ImageClassificationCustomModelTFLiteLoader;
  exports.ImageSegmentation = ImageSegmentation;
  exports.MobilenetTFJS = MobilenetTFJS;
  exports.MobilenetTFJSLoader = MobilenetTFJSLoader;
  exports.MobilenetTFLite = MobilenetTFLite;
  exports.MobilenetTFLiteLoader = MobilenetTFLiteLoader;
  exports.NLClassification = NLClassification;
  exports.ObjectDetection = ObjectDetection;
  exports.QuestionAndAnswer = QuestionAndAnswer;
  exports.SentimentDetection = SentimentDetection;
  exports.TaskModelLoader = TaskModelLoader;
  exports.getAllModelLoaders = getAllModelLoaders;
  exports.getModelLoadersByRuntime = getModelLoadersByRuntime;
  exports.imageClassificationCustomModelTfliteLoader = imageClassificationCustomModelTfliteLoader;
  exports.mobilenetTfjsLoader = mobilenetTfjsLoader;
  exports.mobilenetTfliteLoader = mobilenetTfliteLoader;
  exports.version = version;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=tfjs-tasks.es2017.js.map
